@@ -1,5 +1,6 @@
+from __future__ import absolute_import
+
 import argparse
-import atexit
 import glob
 import os
 import sys
@@ -9,6 +10,7 @@ import re
 import shutil
 import subprocess
 import logging
+from io import open
 
 import lore
 from lore import ansi, env, util
@@ -117,7 +119,7 @@ def api(parsed, unknown):
     
     from hub.listeners.endpoint import EndpointListener
     
-    for path in glob.glob(os.path.join(env.root, 'app', 'api', '*_endpoint.py')):
+    for path in glob.glob(os.path.join(env.root, env.project, 'api', '*_endpoint.py')):
         module = os.path.basename(path)[:-3]
         if sys.version_info.major == 2:
             import imp
@@ -140,7 +142,10 @@ def console(parsed, unknown):
     validate()
     launch_virtual_env()
     check_requirements()
-    
+
+    sys.ps1 = ansi.foreground(env.color, '>>>', readline=True)
+    sys.ps2 = ansi.foreground(env.color, '...', readline=True)
+
     import readline
     history = os.path.join(env.prefix, 'history')
     readline.parse_and_bind('tab: complete')
@@ -156,32 +161,9 @@ def console(parsed, unknown):
     import code
     shell = code.InteractiveConsole(vars)
     
-    sys.ps1 = ansi.foreground(env_color(), '>>>')
-    sys.ps2 = ansi.foreground(env_color(), '...')
-    
     shell.interact('Tip: `help()` or `quit()` to exit')
 
 
-def env_color():
-    return {
-        env.DEVELOPMENT: ansi.GREEN,
-        env.TEST: ansi.BLUE,
-        env.PRODUCTION: ansi.RED
-    }[env.name]
-
-
-def banner():
-    import socket
-    import getpass
-    return '%s in %s %s (Python %s)' % (
-        ansi.foreground(ansi.GREEN, env.project.capitalize()),
-        ansi.foreground(env_color(), env.name),
-        ansi.foreground(ansi.CYAN,
-                        getpass.getuser() + '@' + socket.gethostname()),
-        env.python_version
-    )
-    
-    
 def execute(parsed, unknown):
     validate()
     launch_virtual_env()
@@ -194,7 +176,7 @@ def execute(parsed, unknown):
     print(ansi.success('EXECUTE ') + ' '.join(unknown))
     
     os.environ['PATH'] = os.path.join(env.prefix, 'bin') + ':' + os.environ['PATH']
-    subprocess.check_call(env.args, env=os.environ)
+    subprocess.check_call(unknown, env=os.environ)
 
 
 def init(parsed, unknown):
@@ -232,11 +214,7 @@ def install(parsed, unknown):
 
 def task(parsed, unknown):
     task = unknown[0]
-    try:
-        import app.tasks
-    except:
-        pass
-    print(ansi.success())
+    print(ansi.error() + 'not implemented')
     
     
 def pip(parsed, unknown):
@@ -299,11 +277,12 @@ def install_tensorflow():
     
     print(ansi.success('NATIVE') + ' tensorflow ' + version)
 
+    python_version = str(sys.version_info[0]) + str(sys.version_info[1])
     cached = os.path.join(
         env.pyenv,
         'cache',
         'tensorflow_pkg',
-        'tensorflow-' + version + '*'
+        'tensorflow-' + version + '-cp' + python_version + '*'
     )
     
     paths = glob.glob(cached)
@@ -370,9 +349,9 @@ def build_tensorflow(version):
         'TF_CUDNN_VERSION': '5.1.10',
         'TF_CUDA_CLANG': '/usr/bin/gcc',
         'TF_CUDA_COMPUTE_CAPABILITIES': '3.5,5.2',
+        'TF_NEED_MPI': '0'
     }
     subprocess.check_call(('./configure',), cwd=tensorflow_repo, env=new_env)
-    subprocess.check_call(('./configure',), cwd=tensorflow_repo, env=new_env)
     
     subprocess.check_call((
         'bazel',
@@ -389,14 +368,14 @@ def build_tensorflow(version):
 
 
 def install_python_version():
-    if env.heroku:
+    if env.heroku or env.name == env.PRODUCTION:
         return
         
     if not env.python_version:
         env.set_python_version('.'.join(sys.version_info))
         print(ansi.warning() + ' %s does not exist. Creating with %s' %
               (env.version_path, env.python_version))
-        with open(env.version_path, 'w') as f:
+        with open(env.version_path, 'w', encoding='utf-8') as f:
             f.write(env.python_version + os.linesep)
     
     if not env.pyenv:
@@ -432,24 +411,14 @@ def launch_virtual_env():
     install_python_version()
     
     if env.launched():
-        print(banner())
-        try:
-            import keras
-            def cleanup_tensorflow():
-                # Not strictly necessary, but prevents random gc exception at exit
-                keras.backend.clear_session()
-        
-            atexit.register(cleanup_tensorflow)
-        except:
-            pass
-        
         return
     
-    try:
-        os.unlink(os.path.join(env.pyenv, 'versions', env.project))
-    except OSError as e:
-        if e.errno != os.errno.ENOENT:
-            raise
+    if env.pyenv:
+        try:
+            os.unlink(os.path.join(env.pyenv, 'versions', env.project))
+        except OSError as e:
+            if e.errno != os.errno.ENOENT:
+                raise
 
     if not os.path.exists(env.prefix):
         print(ansi.success('CREATE') + ' virtualenv: %s' % env.project)
@@ -467,14 +436,16 @@ def launch_virtual_env():
 
 
 def install_requirements(args):
-    # Find requirements source
     source = env.requirements
-    if not os.path.exists(source) or args.upgrade:
+    if not os.path.exists(source):
         print(
             ansi.warning() + ' %s is missing. You should add it to version '
                              'control.' % ansi.underline(source)
         )
         source = env.requirements_in
+
+    if args.upgrade:
+        source = env.requirements_in
     
     if not os.path.exists(source):
         sys.exit(
@@ -523,29 +494,7 @@ def freeze_requirements():
         subprocess.check_call(args)
         return freeze_requirements()
     
-    extras = frozen.index(
-        '## The following requirements were added by pip freeze:'
-    )
-    if extras:
-        print(
-            ansi.warning() + ' Packages are installed that are not listed '
-            'in ' + ansi.underline(os.path.basename(env.requirements_in)) +
-            ':' + os.linesep + '  ' +
-            (os.linesep + '  ').join(frozen[extras+1:-1])
-        )
-
-        if vcs:
-            print(
-                ansi.info() + ' The previously listed packages should be '
-                'added unless they are from the following non pypi '
-                'requirements:' + os.linesep + '  ' +
-                (os.linesep + '  ').join(vcs) + os.linesep
-            )
-
-        frozen = frozen[:extras]
-    
-    frozen = [f for f in frozen if not f.startswith('#')]
-    with open(destination, 'w') as f:
+    with open(destination, 'w', encoding='utf-8') as f:
         f.write(
             '# DO NOT EDIT requirements.txt\n# This file is automatically '
             'maintained by `lore install`.\n# Edit requirements.in.txt '
@@ -556,8 +505,8 @@ def freeze_requirements():
 
 
 def split_vcs_lines():
-    with open(env.requirements_in, 'r') as f:
-        lines = f.read().decode('utf-8').split(os.linesep)
+    with open(env.requirements_in, 'r', encoding='utf-8') as f:
+        lines = f.readlines()
     
     vcs = [line for line in lines if
            re.match(r'^(-e )?(git|svn|hg|bzr).*', line)]
@@ -565,24 +514,24 @@ def split_vcs_lines():
         return vcs
     
     if os.path.exists(env.requirements_vcs):
-        with open(env.requirements_vcs, 'r') as f:
-            new = set(f.read().decode('utf-8').split(os.linesep))
+        with open(env.requirements_vcs, 'r', encoding='utf-8') as f:
+            new = set(f.readlines())
             vcs = list(set(vcs).union(new))
     
     lines = list(set(lines) - set(vcs))
-    with open(env.requirements_in, 'w') as f:
-        f.write(os.linesep.join(sorted(lines)))
+    with open(env.requirements_in, 'w', encoding='utf-8') as f:
+        f.write(''.join(sorted(lines)))
     
-    with open(env.requirements_vcs, 'w') as f:
-        f.write(os.linesep.join(sorted(vcs)))
+    with open(env.requirements_vcs, 'w', encoding='utf-8') as f:
+        f.write(''.join(sorted(vcs)))
     return vcs
 
 
 def restore_vcs_lines(vcs):
-    with open(env.requirements_in, 'r') as f:
-        original = f.read().decode('utf-8')
-    with open(env.requirements_in, 'w') as f:
-        f.write(os.linesep.join(vcs) + os.linesep + original)
+    with open(env.requirements_in, 'r', encoding='utf-8') as f:
+        original = f.read()
+    with open(env.requirements_in, 'w', encoding='utf-8') as f:
+        f.write(''.join(vcs) + original)
     os.remove(env.requirements_vcs)
 
 
@@ -611,10 +560,10 @@ def check_requirements():
     if not os.path.exists(env.requirements):
         sys.exit(
             ansi.error() + ' %s is missing. Please run:\n '
-                           '$ lore install\n' % ansi.underline(path)
+                           '$ lore install\n' % ansi.underline(env.requirements)
         )
     
-    with open(env.requirements, 'r') as f:
+    with open(env.requirements, 'r', encoding='utf-8') as f:
         dependencies = f.readlines()
 
     vcs = [d for d in dependencies if
@@ -631,7 +580,7 @@ def check_requirements():
                 error) +
             '\nPlease run:\n $ lore install\n'
         )
-    
+
 
 if __name__ == '__main__':
     try:
@@ -642,5 +591,3 @@ if __name__ == '__main__':
             print(i)
         import traceback
         traceback.print_tb(e[2])
-
-
