@@ -1,13 +1,23 @@
+from __future__ import unicode_literals
+
 import math
 import os
 import re
 import sys
+import logging
+from datetime import timedelta
 
+import inflection
 import numpy
 import pandas
 from smart_open import smart_open
 
 import lore
+import lore.transformers
+from lore.util import timer
+
+
+logger = logging.getLogger(__name__)
 
 
 class Base(object):
@@ -17,6 +27,19 @@ class Base(object):
     initialization before `transform`ing data.
     """
 
+    def __init__(self, column, name=None):
+        super(Base, self).__init__()
+        
+        self.column = column
+        if name:
+            self.name = name
+        else:
+            if isinstance(self.column, lore.transformers.Base):
+                self.name = self.column.name
+            else:
+                self.name = self.column
+            self.name = inflection.underscore(self.__class__.__name__) + '_' + self.name
+        
     def fit(self, data):
         """
         Establishes the encoding for a data set
@@ -27,19 +50,20 @@ class Base(object):
 
     def transform(self, data):
         """
-        :param data: to encode
-        :return: encoded data
+        :param data: DataFrame with column to encode
+        :return: encoded Series
         """
-        return data.apply(self.transform_datum)
+        with timer(('transform %s:' % self.name), logging.DEBUG):
+            return self.series(data).apply(self.transform_datum)
 
-    def reverse_transform(self, data):
+    def reverse_transform(self, series):
         """
         Decodes data
 
         :param data: encoded set to be decoded
         :return: decoded data set
         """
-        return data.apply(self.reverse_transform_datum)
+        return series.apply(self.reverse_transform_datum)
 
     def fit_transform(self, data):
         """
@@ -59,10 +83,10 @@ class Base(object):
         :param addition: uniquely identify this set of fillnas if necessary
         :return: filled data
         """
-        if data.dtype == numpy.object:
-            return data
+        if self.series(data).dtype == numpy.object:
+            return self.series(data)
         
-        return data.fillna(self.missing_value + addition)
+        return self.series(data).fillna(self.missing_value + addition)
         
     def cardinality(self):
         """
@@ -72,23 +96,87 @@ class Base(object):
         :return: the unique number of values this encoding can transform
         """
         pass
+
+    def source_column(self):
+        column = self.column
+        while isinstance(column, lore.transformers.Base):
+            column = column.column
+        return column
+    
+    def series(self, data):
+        if isinstance(self.column, lore.transformers.Base):
+            series = self.column.transform(data)
+        else:
+            series = data[self.column]
+        
+        return series
+        
+
+class Match(Base):
+    """ Provides elementwise comparison of left and right, see also numpy.equal
+    """
+    def __init__(self, column, other, name=None):
+        if not name:
+            column_name = column.name if isinstance(column, lore.transformers.Base) else column
+            other_name = other.name if isinstance(other, lore.transformers.Base) else other
+            name = 'match_' + column_name + '_and_' + other_name
+        super(Match, self).__init__(column=column, name=name)
+        self.other = other
+
+    def fit(self, data):
+        pass
+    
+    def transform(self, data):
+        with timer(('transform %s:' % self.name), logging.DEBUG):
+            return numpy.equal(self.series(data), self.other_series(data)).astype(float)
+    
+    def reverse_transform(self, data):
+        return numpy.full((len(data),), 'LOSSY')
+
+    def cardinality(self):
+        return 2
+
+    def other_series(self, data):
+        if isinstance(self.other, lore.transformers.Base):
+            return self.other.transform(data)
+    
+        return data[self.other]
+
+
+class Continuous(Base):
+    def __init__(self, column, name=None):
+        super(Continuous, self).__init__(column, name)
+
+    def cardinality(self):
+        raise ValueError('Continous values have infinite cardinality')
+
+
+class Pass(Base):
+    def transform(self, data):
+        return self.series(data)
+    
+    def reverse_transform(self, series):
+        return series
     
     
-class Norm(Base):
+class Norm(Continuous):
     """
     Encodes data between 0 and 1. Missing values are encoded to 0, and cannot be
     distinguished from the minimum value observed.
     """
 
-    def __init__(self):
+    def __init__(self, column, name=None):
+        super(Norm, self).__init__(column, name)
         self.__min = float('nan')
         self.__range = float('nan')
         self.missing_value = 0
 
     def fit(self, data):
-        self.__min = float(data.min())
-        self.__range = data.max() - self.__min + 1
-        
+        with timer(('fit %s:' % self.name), logging.DEBUG):
+            series = self.series(data)
+            self.__min = float(series.min())
+            self.__range = series.max() - self.__min + 1
+
     def transform_datum(self, datum):
         if datum is None or (isinstance(datum, float) and math.isnan(datum)):
             return self.missing_value
@@ -98,25 +186,26 @@ class Norm(Base):
     def reverse_transform_datum(self, datum):
         return (datum * self.__range) + self.__min
 
-    def cardinality(self):
-        raise ValueError('Continuous values have infinite cardinality')
-
-
-class LogNorm(Base):
+    
+class LogNorm(Continuous):
     """
     Encodes log(value) between 0 and 1. Missing values are encoded to 0, and
     cannot be distinguished from the minimum value observed.
     """
 
-    def __init__(self):
+    def __init__(self, column, name=None):
+        super(LogNorm, self).__init__(column, name)
         self.__min = float('nan')
         self.__range = float('nan')
         self.missing_value = 0
 
     def fit(self, data):
-        self.__min = math.log(data.min())
-        self.__range = math.log(data.max()) - self.__min
-        
+        with timer(('fit %s:' % self.name), logging.DEBUG):
+            series = self.series(data)
+            self.__min = math.log(series.min())
+            self.__range = math.log(series.max()) - self.__min
+            logger.debug('  %s min: %s range: %s' % (self.name, self.__min, self.__range))
+
     def transform_datum(self, datum):
         if datum is None or (isinstance(datum, float) and math.isnan(datum)):
             return self.missing_value
@@ -126,8 +215,14 @@ class LogNorm(Base):
     def reverse_transform_datum(self, datum):
         return (math.exp(datum) * self.__range) + self.__min
 
-    def cardinality(self):
-        raise ValueError('Continous values have infinite cardinality')
+
+class Boolean(Base):
+    def transform(self, data):
+        series = self.series(data)
+        return series.astype(float)
+    
+    def reverse_transform(self, series):
+        return series.round().astype(bool)
 
 
 class Discrete(Base):
@@ -138,22 +233,33 @@ class Discrete(Base):
     bins + 1.
     """
     
-    def __init__(self, bins=10):
+    def __init__(self, column, name=None, bins=10):
+        super(Discrete, self).__init__(column, name)
         self.__norm = bins - 1
         self.__min = float('nan')
         self.__range = float('nan')
         self.missing_value = self.__norm + 1
+        self.zero = 0.0
     
     def fit(self, data):
-        self.__min = float(data.min())
-        self.__range = data.max() - self.__min
+        with timer(('fit %s:' % self.name), logging.DEBUG):
+            series = self.series(data)
 
+            self.__min = series.min()
+            self.__range = series.max() - self.__min
+            if isinstance(self.__range, timedelta):
+                self.zero = timedelta(0.0)
+                
     def transform_datum(self, datum):
-        if datum is None or (isinstance(datum, float) and math.isnan(datum)):
+        if datum is None or \
+                isinstance(datum, pandas._libs.tslib.NaTType) or \
+                (isinstance(datum, float) and math.isnan(datum)):
             return self.missing_value
+        elif self.__range == 0:
+            return 0
         else:
-            return ((min(self.__range, max(0, datum - self.__min))) / self.__range) * self.__norm
-
+            return (min(self.__range, max(self.zero, datum - self.__min)) * self.__norm) // self.__range
+            
     def reverse_transform_datum(self, datum):
         if datum >= self.missing_value:
             return float('nan')
@@ -163,15 +269,19 @@ class Discrete(Base):
     def cardinality(self):
         return self.__norm + 2
 
+
 class Enum(Base):
     """
     Encodes a number of values from 0 to the max observed. New values that
     exceed previously fit max are capped. Missing values are distinctly encoded.
     """
+    def __init__(self, column, name=None):
+        super(Enum, self).__init__(column, name)
 
     def fit(self, data):
-        self.__max = data.max()
-        self.missing_value = self.__max + 1
+        with timer(('fit %s:' % self.name), logging.DEBUG):
+            self.__max = self.series(data).max()
+            self.missing_value = self.__max + 1
 
     def transform_datum(self, datum):
         if datum is None or (isinstance(datum, float) and math.isnan(datum)):
@@ -189,32 +299,33 @@ class Enum(Base):
         return self.__max + 1
 
 
-class ID(Base):
-    """
-    Encodes distinct values (regardless of type). Values that appear fewer than
+class Unique(Base):
+    """Encodes distinct values. Values that appear fewer than
     minimum_occurrences are mapped to a unique shared encoding to compress the
-    long tail. New values that have not been previously observed will be
+    long tail. New values that were not seen during fit will be
     distinctly encoded from the long tail values.
     """
     
-    def __init__(self, minimum_occurrences=1):
+    def __init__(self, column, name=None, minimum_occurrences=1):
         """
         :param minimum_occurrences: ignore ids with less than this many occurrences
         """
+        super(Unique, self).__init__(column, name)
         self.minimum_occurrences = minimum_occurrences
         self.map = None
         self.inverse = None
         self.missing_value = 1
     
     def fit(self, data):
-        ids = pandas.DataFrame({'id': data})
-        counts = pandas.DataFrame({'n': ids.groupby('id').size()})
-        qualified = counts[counts.n >= self.minimum_occurrences].copy()
-        qualified['encoded_id'] = numpy.arange(len(qualified)) + 1
-        
-        self.map = qualified.to_dict()['encoded_id']
-        self.inverse = {v: k for k, v in self.map.items()}
-        self.missing_value = len(self.map) + 1
+        with timer(('fit unique %s:' % self.name), logging.DEBUG):
+            ids = pandas.DataFrame({'id': self.series(data)})
+            counts = pandas.DataFrame({'n': ids.groupby('id').size()})
+            qualified = counts[counts.n >= self.minimum_occurrences].copy()
+            qualified['encoded_id'] = numpy.arange(len(qualified)) + 1
+            
+            self.map = qualified.to_dict()['encoded_id']
+            self.inverse = {v: k for k, v in self.map.items()}
+            self.missing_value = len(self.map) + 1
     
     def transform_datum(self, datum):
         if datum is None or (isinstance(datum, float) and math.isnan(datum)):
@@ -224,9 +335,9 @@ class ID(Base):
     
     def reverse_transform_datum(self, datum):
         if datum >= self.missing_value:
-            return u'MISSING_VALUE'
+            return 'MISSING_VALUE'
         elif datum == 0:
-            return u'LONG_TAIL'
+            return 'LONG_TAIL'
         else:
             return self.inverse.get(datum)
     
@@ -234,31 +345,28 @@ class ID(Base):
         return len(self.map) + 2
 
 
-class Token(ID):
+class Token(Unique):
     """
     Breaks sentences into individual words, and encodes each word individually,
     with the same properties as the ID encoder.
     """
-    if sys.version_info.major == 2:
-        PUNCTUATION_FILTER = re.compile(u'\W+\s\W+|\W+(\s|$)|(\s|^)\W+', re.UNICODE)
-    elif sys.version_info.major == 3:
-        PUNCTUATION_FILTER = re.compile('\W+\s\W+|\W+(\s|$)|(\s|^)\W+')
+    PUNCTUATION_FILTER = re.compile(r'\W+\s\W+|\W+(\s|$)|(\s|^)\W+', re.UNICODE)
 
-    def __init__(self, sequence_length=10, minimum_occurrences=1):
+    def __init__(self, column, name=None, sequence_length=10, minimum_occurrences=1):
         """
         :param sequence_length: truncates tokens after sequence_length
         :param minimum_occurrences: ignore tokens with less than this many occurrences
         """
-        super(Token, self).__init__(minimum_occurrences=minimum_occurrences)
+        super(Token, self).__init__(column, name, minimum_occurrences)
         self.sequence_length = sequence_length
     
     def fit(self, data):
-        super(Token, self).fit(
-            pandas.Series(
-                [token for sentence in data.apply(Token.tokenize)
-                 for token in sentence]
+        with timer(('fit token %s:' % self.name), logging.DEBUG):
+            tokens = pandas.Series(
+                [token for sentence in self.series(data).apply(Token.tokenize)
+                 for token in sentence], copy=False
             )
-        )
+        super(Token, self).fit(pandas.DataFrame({self.column: tokens}, copy=False))
     
     def transform_datum(self, datum):
         return [
@@ -267,11 +375,16 @@ class Token(ID):
         ]
     
     def reverse_transform_datum(self, datum):
-        return u' '.join([
+        return ' '.join([
             super(Token, self).reverse_transform_datum(token)
             for token in datum
         ])
-    
+
+    def get_token(self, tokens, i):
+        if isinstance(tokens, float) or i >= len(tokens):
+            return self.missing_value
+        return tokens[i]
+
     @staticmethod
     def tokenize(sentence):
         if not sentence:
@@ -290,12 +403,10 @@ class Token(ID):
             
         return re.sub(
             Token.PUNCTUATION_FILTER,
-            u' ',
+            ' ',
             sentence
         ).lower().split()
 
-        return []
-
 
 class Glove(Token):
     """
@@ -304,13 +415,12 @@ class Glove(Token):
     https://blog.keras.io/using-pre-trained-word-embeddings-in-a-keras-model.html
     """
 
-    def __init__(self, sequence_length=10, minimum_occurrences=1):
+    def __init__(self, column, name=None, sequence_length=10, minimum_occurrences=1):
         """
         :param sequence_length: truncates tokens after sequence_length
         :param minimum_occurrences: ignore tokens with less than this many occurrences
         """
-        super(Glove, self).__init__(minimum_occurrences=minimum_occurrences)
-        self.sequence_length = sequence_length
+        super(Glove, self).__init__(column, name, sequence_length, minimum_occurrences)
 
 
     def __getstate__(self):
@@ -332,8 +442,8 @@ class Glove(Token):
         self.inverse = {}
 
         path = os.path.join(lore.env.models_dir, 'encoders', 'glove.6B.%dd.txt.gz' % self.dimensions)
-        local = app.data.download(path)
-        with app.timer("Loading GloVe parameters: %s" % (local)):
+        local = lore.io.download(path)
+        with timer("Loading GloVe parameters: %s" % (local), logging.DEBUG):
             for line in smart_open(local):
                 values = line.split()
                 word = values[0]
@@ -373,7 +483,8 @@ class MiddleOut:
         self.depth = depth
     
     def fit(self, data):
-        pass
+        with timer("fit middle out: ", logging.DEBUG):
+            pass
     
     def transform(self, x):
         max_seq = len(x)
@@ -425,7 +536,8 @@ class Days:
 
     def fit(self, days):
         """Fit encoder to days array."""
-        self.max_days = numpy.max(days)
+        with timer("fit days: ", logging.DEBUG):
+            self.max_days = numpy.max(days)
 
     def transform(self, days):
         """Transform days array."""
@@ -464,16 +576,17 @@ class Single:
         Return:
             a dict mapping ids to encoded ids
         """
-        df = pandas.DataFrame({"ids": ids})
-
-        counts = df.groupby("ids").size()
-        counts = pandas.DataFrame({"n": counts})
-
-        top = counts[counts.n >= self.minimum_occurrences].copy()
-        top['encoded_id'] = numpy.arange(len(top)) + 2
-        id_map = top.to_dict()['encoded_id']
-
-        self.map = id_map
+        with timer("fit single:", logging.DEBUG):
+            df = pandas.DataFrame({"ids": ids})
+    
+            counts = df.groupby("ids").size()
+            counts = pandas.DataFrame({"n": counts})
+    
+            top = counts[counts.n >= self.minimum_occurrences].copy()
+            top['encoded_id'] = numpy.arange(len(top)) + 2
+            id_map = top.to_dict()['encoded_id']
+    
+            self.map = id_map
 
     def transform(self, ids):
         """Transforms ids into encoded IDs.
